#!/usr/bin/env python
#
# Problem EPage 411
#
# PYTHONPATH=../../BookCode:$PYTHONPATH
#
import numpy as np; from numpy import asarray, cos, sin, pi

from powell import powell

# Make sure everything is in base SI units: 
b = 4 # meters 
P = 50 * 1.e3 # Newtons 
E = 200 * 1.e9 # Pa 

def objective_fn(x):
    """
    The function we want to minimize.
    """
    theta, A = x
    return ( b * A ) / cos(theta) 

def constraint_fns(x): 
    """
    A tuple of constraints we want to be <= 0.

    A >= 0 
    theta >= 0 
    theta <= pi/2
    """
    theta, A = x
    sigma = P / ( 2 * A * sin(theta) ) 
    delta = ( P * b ) / ( 2 * E * A * sin(2*theta) * sin(theta) )
    return ( -A, -theta, theta - pi/2, sigma - 150 * 1.e6, delta - 5 * 1.e-3 )

def lagrangian_function(lam): 
    """
    Given a value of lambda returns a function of x that we can minimize with a minimization routine.
    """
    return lambda _ : F(_,lam)

def F(x,lam): 
    """
    Given lambda returns the value of the objective function 
    """
    objective = objective_fn(x)
    penalties = map( lambda _: max(_,0)**2, constraint_fns(x) )
    penalty = sum( penalties ) 
    return objective + lam * penalty 

# Get a valid initial guess for the numerical value of A:
# 
theta = pi/6.
C1 = P/(2 * sin(theta))
t = 150 * 1.e6
A1_ec = C1/t
print "First constraint (without A)= %10.6f, target= %10.6f, A_ec= %10.6f" % (C1, t, A1_ec)

C2 = (P * b)/( 2 * E * sin(2*theta) * sin(theta) )
t = 5 * 1.e-3 
A2_ec = C2/t
print "Second constraint (without A)= %10.6f, target= %10.6f, A_ec= %10.6f" % (C2, t, A2_ec)

A = 0.5 * ( A1_ec + A2_ec ) 
print "Initial guess for A= %10.6f" % A

# Start with an initial value of a Lagrangian value of lambda: 
lam = 1.e5
xStart = asarray( [theta,A] )
x, numIter = powell( lagrangian_function(lam), xStart, 0.00001 ) 
print "Constraint Lambda= ", lam
print "Minimium point= ", x
print "Constraints satisfied (all negative or zero)? ", map( lambda _: round(_,4), constraint_fns(x) )
print "Function value= ", F(x,lam)

# Increase lambda: 
lam *= 10.
x, numIter = powell( lagrangian_function(lam), x, 0.01 ) 
print "Constraint Lambda= ", lam
print "Minimium point= ", x
print "Constraints satisfied (all negative or zero)? ", map( lambda _: round(_,4), constraint_fns(x) )
print "Function value= ", F(x,lam)





